// GSAgolfCPAPI.cpp : This file contains the 'main' function. Program execution begins and ends there.
//

#include <iostream>
#include <windows.h>
#include <stdio.h>
#include <fstream>
#include <iostream>
#include <stdlib.h>
#include <memory.h>
#include <conio.h>


static const LONG BufferSize = 100;
static const LONG Buffers = 13;

static const char MemoryName[] = "GSASDK2508";

static char(*Memory)[Buffers][BufferSize];

static const char WriterSemaphoreName[] = "Writer Semaphore";
static const char ReaderSemaphoreName[] = "Reader Semaphore";

static HANDLE hWriterSemaphore;
static HANDLE hReaderSemaphore;

static HANDLE hMemory;
struct launchParamsStruc {

	bool bSystemReady;// system ready flag indicates if golf sim is running and ready 
	bool bOkToLaunch; // E6 software should poll this flag. If set then E6 should cal trajectory and launch ball with the set ball and club track parameters
	float ballSpeed;  // ball speed in mph
	float ballPath;   // ball path in degrees left or right + degrees = ball path is right, - degrees ball path is left
	float ballAngle;  // vertical launch angle 
	float clubSpeed;  // club speed in mph
	float clubPath;   // club path in degrees left or right + degrees = club path is right, - degrees club path is left
	float clubFaceAngle;    // raw club face angle (not relative to club path)  (+ = open, - = closed)
	float distanceToTarget; // current ball distance to pin in yards
	bool bLeftHanded;       // when set, player is left handed, otherwise right handed
	bool bPutting;	        // player has selectd putter
	int club;			    // club type (see above defintion)
	int currentSurfaceLie;  // current ball lie surface type (see above definition)
	int spin;               // rpm - if 0 then ignore
	float spinAxis;           // ball spin axis tilt is degrees left or right (+ = right, - = left)
	float spinTilt;
	int sideSpin;

};
launchParamsStruc launchParams;




// club types

#define CLUB_IRON_1 0
#define CLUB_IRON_2 1
#define CLUB_IRON_3 2
#define CLUB_IRON_4 3
#define CLUB_IRON_5 4
#define CLUB_IRON_6 5
#define CLUB_IRON_7 6
#define CLUB_IRON_8 7
#define CLUB_IRON_9 8

#define CLUB_PITCHING_WEDGE 9
#define CLUB_APPROACH 10
#define CLUB_SAND_WEDGE 11
#define CLUB_LOB_WEDGE 12
#define CLUB_GAP_WEDGE 23
#define CLUB_DRIVER 13
#define CLUB_HYBRID_2 14
#define CLUB_HYBRID_3 15

#define CLUB_HYBRID_5 16

#define CLUB_WOOD_3 17
#define CLUB_WOOD_5 18
#define CLUB_WOOD_7 19
#define CLUB_WOOD_9 20
#define CLUB_PUTTER 21
#define CLUB_HYBRID_4 22


#define SURFACE_GREEN		1
#define SURFACE_FAIRWAY		2
#define SURFACE_ROUGH		3
#define SURFACE_SAND		4
#define SURFACE_TEE			5
#define SURFACE_FOOTPATH	6
#define SURFACE_UNKNOWN		10
#define SURFACE_PIN_HOLE	11
#define SURFACE_BRACKLE		12
#define SURFACE_HIROUGH		13
#define SURFACE_ON_TEE_BOX	14
#define SURFACE_FRINGE		15


bool bSMopen = false; // general flag to indicate that the shared memory file has been opened
bool b64 = true; // set true if your app is a 64 bit app, fallse if a 32 bit app
char s[100];

// ************************* trajectoryParamsReady *******************************************
//   reads status of the CP trajectory parameters from shared memory

//   Your program should poll this bit in the shared memory to check if valid swing
//   data is present regularly 
//   ie at least once every 0.2 seconds
//  


bool trajectoryParamsReady(void)
{
	return ((*Memory)[0][0]);
}

// ************************* getTrajectoryParams *******************************************
//   reads the GSA CP trajectory parameters from shared memory
//   all data required to launch a ball are read from the shared memory and stored in the launchParams data structure.
//   The club-face angle supplied from the CP is absolute and not relative to the club path so you should adjust accordingly with the 
//   club-head path to find the true relative club-face angle. This should then be used to calculate the ball side spin (or axis tilt)
//   which in turn is used to calculate the ball flight trajectory curve. ie draws,fades,slices and hooks, 

void trajectoryParamsReadyReset(void)
{
	(*Memory)[0][0] = 0;
}

bool getTrajectoryParams(void)
{

	/*
	memory buffer data structure

	Buffer No.  BufferIndex  Content

	0			0	int				0 = launch data not ready, 1 = Launch data Ready
	1			0 	float			Ball Speed in mph format nn.nn
	2			0	float			Ball Path in +/-degrees format +-nn.nn
	3			0	float			Ball Vertical Launch Angle format nn.nn
	4			0	float			Clubhead speed in m/s format nn.nn
	5			0	float			Clubhead path in +/-degrees format +-nn.nn
	6			0	float 			Clubface angle in +/-degrees format +-nn.nn
	7			0	int				Club type : see club constants
	8			0					na
	9			0					na
	10			0	int				Ball back spin
	11			0	float			Spin axis
	12			0	int 			Side spin

	*/



	int n = 0;
	char s[100];

	for (int i = 0; i <= 13; ++i)
	{



		switch (i)
		{
		case 0:
			launchParams.bOkToLaunch = atoi((*Memory)[i]);
			s[0] = (*Memory)[i][6];
			break;
		case 1:
			launchParams.ballSpeed = atof((*Memory)[i]);

			break;
		case 2:
			launchParams.ballPath = atof((*Memory)[i]);

			break;
		case 3:
			launchParams.ballAngle = atof((*Memory)[i]);

			break;
		case 4:
			launchParams.clubSpeed = atof((*Memory)[i]);

			break;
		case 5:
			launchParams.clubPath = atof((*Memory)[i]);

			break;
		case 6:
			launchParams.clubFaceAngle = atof((*Memory)[i]);

			break;
		case 7:
			launchParams.club = atoi((*Memory)[i]);

			break;

		case 10:
			launchParams.spin = atoi((*Memory)[i]);

			break;
		
		case 12:
			launchParams.sideSpin = atoi((*Memory)[i]);

			break;

		}



	}
	return true;
}
void closeSharedMemory(void)
{
	UnmapViewOfFile(Memory);
	CloseHandle(hWriterSemaphore);
	CloseHandle(hReaderSemaphore);
	bSMopen = false;
}


bool openSharedMemory(void)
{
	bSMopen = false;
	hWriterSemaphore = CreateSemaphore(NULL, Buffers, Buffers, WriterSemaphoreName);

	if (!hWriterSemaphore)
		return false;

	hReaderSemaphore = CreateSemaphore(NULL, 0, Buffers, ReaderSemaphoreName);


	if (!hReaderSemaphore)
	{
		CloseHandle(hWriterSemaphore);
		return false;
	}

	if (b64)
		hMemory = CreateFileMapping((HANDLE)0xFFFFFFFFFFFFFFFF, NULL, PAGE_READWRITE, 0
			, sizeof(char[Buffers][BufferSize]), MemoryName);
	else
		hMemory = CreateFileMapping((HANDLE)0xFFFFFFFF, NULL, PAGE_READWRITE, 0
			, sizeof(char[Buffers][BufferSize]), MemoryName);


	if (!hMemory)
	{
		CloseHandle(hReaderSemaphore);
		CloseHandle(hWriterSemaphore);
		return false;
	}



	Memory = (char(*)[Buffers][BufferSize])MapViewOfFile(hMemory
		, FILE_MAP_WRITE, 0, 0, sizeof(char[Buffers][BufferSize]));


	if (!Memory)
	{
		CloseHandle(hReaderSemaphore);
		CloseHandle(hWriterSemaphore);
		return false;
	}

	bSMopen = true;
	return true;
}

int shotNum = 1;


int main()
{
	std::cout << "GSA CP API : press any key to exit \n";
	std::cout << "GSA CP API : opening shared memory file \n";
	if(!openSharedMemory())
		std::cout << "Cannot open file \n";
	else
	{
		std::cout << "GSA CP API : shared memory file opened \n";
		std::cout << "GSA CP API : waiting for data \n";
		while (1)
		{
			if (trajectoryParamsReady())
			{
				trajectoryParamsReadyReset();
				
				getTrajectoryParams();
				std::cout << " \n";
				sprintf_s(s, "Shot %d \n", shotNum);
				std::cout << s;
				std::cout << " \n";
				sprintf_s(s, "Ball speed: %0.2f mph \n", launchParams.ballSpeed);
				std::cout << s;
				sprintf_s(s, "Ball HLA: %0.2f degrees \n", launchParams.ballPath);
				std::cout << s;
				sprintf_s(s, "Ball VLA: %0.2f degrees \n", launchParams.ballAngle);
				std::cout << s;
				sprintf_s(s, "Club speed: %0.2f mph \n", launchParams.clubSpeed);
				std::cout << s;
				sprintf_s(s, "Club path: %0.2f degrees \n", launchParams.clubPath);
				std::cout << s;
				sprintf_s(s, "Club face: %0.2f degrees \n", launchParams.clubFaceAngle);
				std::cout << s;
				sprintf_s(s, "Club type: %d \n", launchParams.club);
				std::cout << s;
				sprintf_s(s, "Ball back spin: %d rpm \n", launchParams.spin);
				std::cout << s;
				sprintf_s(s, "Ball side spin: %d rpm \n", launchParams.sideSpin);
				std::cout << s;

				std::cout << " \n";
				shotNum++;
				std::cout << "GSA CP API : waiting for next launch data \n";
				
			}
			if (_kbhit())
				break;


		}
	}
	std::cout << "GSA CP API : press any key to exit \n";
	if (_kbhit())
		return 0;

}

// Run program: Ctrl + F5 or Debug > Start Without Debugging menu
// Debug program: F5 or Debug > Start Debugging menu

